'use strict'
// @ts-check

const _ = require('lodash')
const process = require('process')
const Logger = require('@fizz.js/node-logger')
const ErrorReporter = require('@fizz.js/node-error-reporter/libs')

/**
 *
 *
 * @class ServerHandler
 */
class ServerHandler {
  /**
   *
   * @see http://pm2.keymetrics.io/docs/usage/signals-clean-restart/#graceful-start
   * @returns {void}
   * @memberof ServerHandler
   */
  async onListening() {
    try {
      const isReady = _.has(process, 'send') && _.isFunction(_.get(process, 'send'))
      if (isReady) {
        process.send('APP IS READY...')
      }
    } catch (error) {
      Logger.error(error)
    }
  }

  /**
   *
   *
   * @param {Error} err
   * @returns {void}
   * @memberof ServerHandler
   */
  async onError(err) {
    try {
      await ErrorReporter.handleError(err, 'error', true).catch(e => Logger.warn(e))
      if (_.get(err, 'syscall') !== 'listen') {
        throw err
      }

      switch (_.get(err, 'code')) {
        case 'EACCES':
        case 'EADDRINUSE':
          process.exit(1)

        default:
          throw err
      }
    } catch (error) {
      Logger.error(error)
    }
  }
}

module.exports = new ServerHandler()
